c"---------------------------------------------------------------------"
c"                                                                     "
c" Author:      S.D. Sudhoff                                           "
c"              Purdue University                                      "
c"              School of Electrical and Computer Engineering          "
c"              1285 Electrical Engineering Bulding                    "
c"              West Lafayette, IN 47906                               "
c"              Phone: 765-494-3246                                    "
c"              Fax: 765-494-0676                                      "
c"              E-mail: sudhoff@ecn.purdue.edu                         "
c" Version:     1.0                                                    "
c"                                                                     "
c"---------------------------------------------------------------------"
c"                                                                     "
c" Subroutine: LCC                                                     "
c"                                                                     "
c" Description: educed order model of a three phase synchronous        "
c"              machine connected to a line commutated converter.      "
c"                                                                     "
c"                                INPUTS                               " 
c"                                                                     "
c"    rs         -     stator resistance (Ohms)                        " 
c"    xq         -     q- axis subtransient reactance (Ohms)           " 
c"    xd         -     d- axis subtransient reactance (OHMS)           " 
c"    siqpp      -     q- axis subtransient flux linkage (V-sec)       " 
c"    sidpp      -     d- axis subtransient flux linkage (V-sec)       "
c"    idc        -     dc current out of the rectifier (A)             "
c"    wrn        -     normalized rotor speed                          " 
c"    alphamode  -     logic switch to indcate use as a controlled     "
c"                     rectifier.                                      "
c"    fireangle  -     commanded firing angle relative to rotor        " 
c"                     position (rad)                                  " 
c"                                                                     "
c"                               OUTPUTS                               " 
c"                                                                     "
c"    vdcbar     -     average dc voltage across the rectifier (V)     "
c"    iqbar      -     average q-axis current into machine stator (A)  "
c"    idbar      -     average d-axis current into machine stator (A)  "
c"    xct        -     transient commutating reactance (Ohms)          "
c"    beta       -     firing angle relative to rotor position (Rad)   " 
c"                                                                     "
c"---------------------------------------------------------------------"

       SUBROUTINE LCC(rs,xq,xd,siqpp,sidpp,idc,wrn,
     &                alphamode,fireangle,  
     &                vdcbar,iqbar,idbar,betaout,xct)

       DOUBLE PRECISION rs,xq,xd,xqmxd,xqpxd,siqpp,sidpp,idc
       DOUBLE PRECISION betamin,wrn
       DOUBLE PRECISION fireangle,vdcbar,iqbar,idbar
       LOGICAL alphamode
       DOUBLE PRECISION xc,xct
       DOUBLE PRECISION u,g,u1,u2
       DOUBLE PRECISION k
       DOUBLE PRECISION fbmin,beta1,beta2
       DOUBLE PRECISION beta,alpha
       DOUBLE PRECISION eq,ed,E,phi 
       DOUBLE PRECISION dci,sq,sd,iqcond,idcond,iqcom,idcom
       DOUBLE PRECISION root3,twothirds,volfac,curfac
       DOUBLE PRECISION pi,piover6,piover3,twopiover3  
       DOUBLE PRECISION iqsr,idsr

c      constants
       PARAMETER(root3=1.7320508)
       PARAMETER(pi=3.1415927)
       PARAMETER(piover6=pi/6.0)  
       PARAMETER(piover3=pi/3.0)
       PARAMETER(twopiover3=2.0*pi/3.0)
       PARAMETER(twothirds=2.0/3.0)
       PARAMETER(volfac=3.0*root3/pi)
       PARAMETER(curfac=2.0*root3/pi)
    
       
       COMMON /STUFF/beta,dci,k,sq,sd,xqmxd,xqpxd
       COMMON /RCN/u,eq,ed
      
       dci=idc
       sq=siqpp
       sd=sidpp
       xqmxd=xq-xd
       xqpxd=xq+xd


c      compute back emf
       eq=wrn*sidpp
       ed=-wrn*siqpp
       E=sqrt(eq*eq+ed*ed)
       phi=atan2(ed,eq)
 
c      compute minimum beta
       beta1=-pi
       beta2=pi
       betamin=0.0
       fbmin= root3*(siqpp*cos(betamin)+sidpp*sin(betamin))+
     &        2.0*idc*xqmxd*sin(2.0*betamin-piover3)
1000   IF (ABS(beta1-beta2) .GT. 0.001) THEN
	  betamin =  (beta1+beta2)/2.0
	  fbmin   =  root3*(siqpp*cos(betamin)+sidpp*sin(betamin))+
     &               2.0*idc*xqmxd*sin(2.0*betamin-piover3)
	  IF (fbmin .LE. 0.0) THEN
	     beta1=betamin
	  ELSE
	     beta2=betamin
	  ENDIF
	  GOTO 1000 
       END IF
 
c      determine beta and alpha
       IF (alphamode) THEN
	  beta=fireangle+betamin
       ELSE
	  beta=max(fireangle,betamin)
	  betaout=beta
       END IF
       alpha=beta-phi  


c      determine K
       K=root3*(-siqpp*sin(beta)+sidpp*cos(beta))+
     &   (-xqmxd*cos(2.0*beta+twopiover3)-0.5*xqpxd)*idc


c      determine commutation angle
       u1=0.0
       u2=piover3
       u=0.0
       g= K-root3*(-siqpp*sin(beta+u)+sidpp*cos(beta+u))-
     &    (xqmxd*cos(2.0*(beta+u)-twopiover3)+0.5*xqpxd)*idc
1050   IF (ABS(u1-u2) .GT. 0.001) THEN
	  u=0.5*(u1+u2)
	  g= K-root3*(-siqpp*sin(beta+u)+sidpp*cos(beta+u))-
     &       (xqmxd*cos(2.0*(beta+u)-twopiover3)+0.5*xqpxd)*idc
	  IF (g .LE. 0.0) THEN
	     u1=u
	  ELSE
	     u2=u
	  ENDIF
	  GOTO 1050
       END IF

c      compute commutating reactance 
       xc=0.5*xqpxd-xqmxd*sin(2.0*beta+piover6)

c      determine transient commutating reactance
       xct=xqpxd-xqmxd*sin(2.0*beta-piover6)

c      compute average dc voltage
       vdcbar=volfac*E*cos(alpha)-(wrn*xc/piover3+2.0*rs)*idc

c      compute conduction currents
       iqcond=curfac*idc*(cos(beta+twopiover3)-cos(beta+u+piover3))
       idcond=curfac*idc*(sin(beta+twopiover3)-sin(beta+u+piover3))

c      compute commutation currents
       iqcom=(u/(12.0*piover3))*
     &       (iqsr(0.0)+4.0*iqsr(0.25*u)+2.0*iqsr(0.5*u)+
     &        4.0*iqsr(0.75*u)+iqsr(u))
       idcom=(u/(12.0*piover3))*
     &       (idsr(0.0)+4.0*idsr(0.25*u)+2.0*idsr(0.5*u)+
     &        4.0*idsr(0.75*u)+idsr(u))


c      compute average q- and d-axis currents
       iqbar=iqcond+iqcom
       idbar=idcond+idcom

c      compute instantaneous waveforms


       END


       SUBROUTINE INSTANT(instantmode,xq,xd,rs,wrn,idc,pidcbar,thetar,
     &                    ia,ib,ic,va,vb,vc,vdc,te)

       LOGICAL instantmode
       DOUBLE PRECISION xq,xd,rs,wrn,idc,pidcbar,thetar
       DOUBLE PRECISION thetarhat
       DOUBLE PRECISION ias,dias
       DOUBLE PRECISION ia,ib,ic,dia,dib,dic
       DOUBLE PRECISION iq,id,diq,did,va,vb,vc,vdc,te
       DOUBLE PRECISION vq,vd
       DOUBLE PRECISION cost,sint,costm120,costp120,sintm120,sintp120
       DOUBLE PRECISION cosr,sinr,cosrm120,cosrp120,sinrm120,sinrp120
	
       DOUBLE PRECISION u,eq,ed
       DOUBLE PRECISION beta,dci,k,sq,sd,xqmxd,xqpxd
       DOUBLE PRECISION root3,piover3,pi,twothirds

       COMMON /RCN/u,eq,ed
       COMMON /STUFF/beta,dci,k,sq,sd,xqmxd,xqpxd

       PARAMETER(root3=1.7320508)
       PARAMETER(pi=3.1415927)
       PARAMETER(piover3=pi/3.0)
       PARAMETER(twothirds=2.0/3.0)

       dci=idc

       IF (instantmode) THEN

	  thetarhat=MOD(thetar-piover3-beta,piover3)
       
	  IF (thetarhat .LT. 0.0) THEN
	     thetarhat=thetarhat+piover3
	  END IF

	  IF ((thetarhat .LT. 0.0) .OR.
     &        (thetarhat .GT. piover3)) THEN
	      PRINT *,'thetarhat error'
	  END IF


	  IF (thetarhat .LE. u) THEN
	     ia= ias(thetarhat)
	     ib=-idc-ia
	     ic= idc
	     dia= dias(ia,thetarhat,pidcbar)
	     dib=-pidcbar-dia
	     dic= pidcbar
	  ELSE
	     ia=0.0
	     ib=-idc
	     ic= idc
	     dia=0.0
	     dib=-pidcbar
	     dic= pidcbar
	  END IF

	  cost=cos(thetarhat+piover3+beta)
	  sint=sin(thetarhat+piover3+beta)
	  costm120=0.5*(-cost+root3*sint)
	  costp120=0.5*(-cost-root3*sint)
	  sintm120=0.5*(-sint-root3*cost)
	  sintp120=0.5*(-sint+root3*cost)

	  iq=twothirds*(ia*cost+ib*costm120+ic*costp120)
	  id=twothirds*(ia*sint+ib*sintm120+ic*sintp120)

	  te=iq*(xd*id+sd)-id*(xq*iq+sq)

	  diq=twothirds*(dia*cost+dib*costm120+dic*costp120
     &                   -ia*sint -ib*sintm120 -ic*sintp120)
	  did=twothirds*(dia*sint+dib*sintm120+dic*sintp120
     &                   +ia*cost +ib*costm120 +ic*costp120)

	  vq=rs*iq+wrn*( xd*id+xq*diq)+eq
	  vd=rs*id+wrn*(-xq*iq+xd*did)+ed

	  vdc=root3*(sin(thetarhat+piover3+beta)*vq-
     &               cos(thetarhat+piover3+beta)*vd)
 
	  cosr=cos(thetar)
	  sinr=sin(thetar)
	  cosrm120=0.5*(-cosr+root3*sinr)
	  cosrp120=0.5*(-cosr-root3*sinr)
	  sinrm120=0.5*(-sinr-root3*cosr)
	  sinrp120=0.5*(-sinr+root3*cosr)

	  va=vq*cosr+vd*sinr 
	  vb=vq*cosrm120+vd*sinrm120
	  vc=vq*cosrp120+vd*sinrp120

	  ia=iq*cosr+id*sinr
	  ib=iq*cosrm120+id*sinrm120
	  ic=iq*cosrp120+id*sinrp120

       END IF

       END 

       DOUBLE PRECISION FUNCTION iqsr(thetarhat)
       DOUBLE PRECISION thetarhat,ias

       DOUBLE PRECISION beta,dci,k,sq,sd,xqmxd,xqpxd
       COMMON /STUFF/beta,dci,K,sq,sd,xqmxd,xqpxd

       DOUBLE PRECISION twooverroot3,pi,piover3
       PARAMETER(pi=3.1415927)
       PARAMETER(piover3=pi/3.0)
       PARAMETER(twooverroot3=2.0/1.7320508)
       
       iqsr=twooverroot3*(-ias(thetarhat)*sin(thetarhat+beta)-
     &                     dci*sin(thetarhat+piover3+beta))

       END

       DOUBLE PRECISION FUNCTION idsr(thetarhat)
       DOUBLE PRECISION thetarhat,ias

       DOUBLE PRECISION beta,dci,k,sq,sd,xqmxd,xqpxd
       COMMON /STUFF/beta,dci,K,sq,sd,xqmxd,xqpxd

       DOUBLE PRECISION twooverroot3,pi,piover3
       PARAMETER(pi=3.1415927)
       PARAMETER(piover3=pi/3.0)
       PARAMETER(twooverroot3=2.0/1.7320508)

       idsr=twooverroot3*(ias(thetarhat)*cos(thetarhat+beta)+
     &                    dci*cos(thetarhat+piover3+beta))
       
       END

       
       DOUBLE PRECISION FUNCTION ias(thetarhat)
       DOUBLE PRECISION thetarhat

       DOUBLE PRECISION beta,dci,k,sq,sd,xqmxd,xqpxd
       COMMON /STUFF/beta,dci,k,sq,sd,xqmxd,xqpxd
       
       DOUBLE PRECISION root3,pi,twopiover3
       PARAMETER(root3=1.7320508)
       PARAMETER(pi=3.1415927)
       PARAMETER(twopiover3=2.0*pi/3.0)
       
       ias=(k+root3*(sq*sin(thetarhat+beta)-sd*cos(thetarhat+beta))
     &    -( xqmxd*cos(2.0*(thetarhat+beta)-twopiover3)+0.5*xqpxd)*dci)/
     &     (-xqmxd*cos(2.0*(thetarhat+beta))+xqpxd)

       END

       DOUBLE PRECISION FUNCTION dias(ia,thetarhat,pidc)
       DOUBLE PRECISION thetarhat,ia,pidc

       DOUBLE PRECISION beta,dci,k,sq,sd,xqmxd,xqpxd
       COMMON /STUFF/beta,dci,k,sq,sd,xqmxd,xqpxd
       
       DOUBLE PRECISION root3,pi,twopiover3,piover3
       PARAMETER(root3=1.7320508)
       PARAMETER(pi=3.1415927)
       PARAMETER(piover3=pi/3.0)
       PARAMETER(twopiover3=2.0*pi/3.0)
 
       dias=(root3*(sq*cos(thetarhat+beta)+sd*sin(thetarhat+beta))+
     &       2.0*xqmxd*sin(2.0*(thetarhat+beta)-twopiover3)*dci-
     &      ia*2.0*xqmxd*sin(2.0*(thetarhat+beta)) 
     &    - ( xqmxd*cos(2.0*(thetarhat+beta)-twopiover3)
     &       +0.5*xqpxd)*pidc)/
     &      (-xqmxd*cos(2.0*(thetarhat+beta))+xqpxd)
	   
       END
